@@ -25,11 +25,14 @@ use crate::ui::Ui;
 
 /// Touch up the content changes in a revision with a diff editor
 ///
-/// With the `-r` option, which is the default, starts a diff editor (`meld` by
-/// default) on the changes in the revision.
+/// With the `-r` option, which is the default, starts a [diff editor] on the
+/// changes in the revision.
 ///
-/// With the `--from` and/or `--to` options, starts a diff editor comparing the
-/// "from" revision to the "to" revision.
+/// With the `--from` and/or `--to` options, starts a [diff editor] comparing
+/// the "from" revision to the "to" revision.
+///
+/// [diff editor]:
+///     https://martinvonz.github.io/jj/latest/config/#editing-diffs
 ///
 /// Edit the right side of the diff until it looks the way you want. Once you
 /// close the editor, the revision specified with `-r` or `--to` will be
@@ -81,7 +84,7 @@ pub(crate) fn cmd_diffedit(
         base_commits = target_commit.parents();
         diff_description = "The diff initially shows the commit's changes.".to_string();
     };
-    workspace_command.check_rewritable([&target_commit])?;
+    workspace_command.check_rewritable([target_commit.id()])?;
 
     let diff_editor = workspace_command.diff_editor(ui, args.tool.as_deref())?;
     let mut tx = workspace_command.start_transaction();
