@@ -14,7 +14,9 @@
 
 //! This file contains the internal implementation of `run`.
 
-use crate::cli_util::{resolve_multiple_nonempty_revsets, CommandHelper, RevisionArg};
+use itertools::Itertools as _;
+
+use crate::cli_util::{CommandHelper, RevisionArg};
 use crate::command_error::{user_error, CommandError};
 use crate::ui::Ui;
 
@@ -48,7 +50,10 @@ pub struct RunArgs {
 
 pub fn cmd_run(ui: &mut Ui, command: &CommandHelper, args: &RunArgs) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
-    let _resolved_commits = resolve_multiple_nonempty_revsets(&args.revisions, &workspace_command)?;
+    let _resolved_commits: Vec<_> = workspace_command
+        .parse_union_revsets(&args.revisions)?
+        .evaluate_to_commits()?
+        .try_collect()?;
     // Jobs are resolved in this order:
     // 1. Commandline argument iff > 0.
     // 2. the amount of cores available.
