@@ -13,8 +13,9 @@
 // limitations under the License.
 
 use itertools::Itertools as _;
-use jj_lib::commit::Commit;
+use jj_lib::commit::{Commit, CommitIteratorExt};
 use jj_lib::matchers::Matcher;
+use jj_lib::merged_tree::MergedTree;
 use jj_lib::object_id::ObjectId;
 use jj_lib::repo::Repo;
 use jj_lib::revset;
@@ -53,15 +54,19 @@ pub(crate) struct SquashArgs {
     /// Revision to squash into its parent (default: @)
     #[arg(long, short)]
     revision: Option<RevisionArg>,
-    /// Revision to squash from (default: @)
+    /// Revision(s) to squash from (default: @)
     #[arg(long, conflicts_with = "revision")]
-    from: Option<RevisionArg>,
+    from: Vec<RevisionArg>,
     /// Revision to squash into (default: @)
     #[arg(long, conflicts_with = "revision", visible_alias = "to")]
     into: Option<RevisionArg>,
     /// The description to use for squashed revision (don't open editor)
     #[arg(long = "message", short, value_name = "MESSAGE")]
     message_paragraphs: Vec<String>,
+    /// Use the description of the destination revision and discard the
+    /// description(s) of the source revision(s)
+    #[arg(long, short, conflicts_with = "message_paragraphs")]
+    use_destination_message: bool,
     /// Interactively choose which parts to squash
     #[arg(long, short)]
     interactive: bool,
@@ -83,11 +88,14 @@ pub(crate) fn cmd_squash(
 
     let mut sources: Vec<Commit>;
     let destination;
-    if args.from.is_some() || args.into.is_some() {
-        sources = workspace_command
-            .parse_revset(args.from.as_ref().unwrap_or(&RevisionArg::AT))?
-            .evaluate_to_commits()?
-            .try_collect()?;
+    if !args.from.is_empty() || args.into.is_some() {
+        sources = if args.from.is_empty() {
+            workspace_command.parse_revset(&RevisionArg::AT)?
+        } else {
+            workspace_command.parse_union_revsets(&args.from)?
+        }
+        .evaluate_to_commits()?
+        .try_collect()?;
         destination =
             workspace_command.resolve_single_rev(args.into.as_ref().unwrap_or(&RevisionArg::AT))?;
         if sources.iter().any(|source| source.id() == destination.id()) {
@@ -108,13 +116,13 @@ pub(crate) fn cmd_squash(
         destination = parents.pop().unwrap();
     }
 
-    let matcher = workspace_command.matcher_from_values(&args.paths)?;
+    let matcher = workspace_command
+        .parse_file_patterns(&args.paths)?
+        .to_matcher();
     let diff_selector =
         workspace_command.diff_selector(ui, args.tool.as_deref(), args.interactive)?;
     let mut tx = workspace_command.start_transaction();
     let tx_description = format!("squash commits into {}", destination.id().hex());
-    let description = (!args.message_paragraphs.is_empty())
-        .then(|| join_message_paragraphs(&args.message_paragraphs));
     move_diff(
         ui,
         &mut tx,
@@ -123,14 +131,42 @@ pub(crate) fn cmd_squash(
         &destination,
         matcher.as_ref(),
         &diff_selector,
-        description,
-        args.revision.is_none() && args.from.is_none() && args.into.is_none(),
+        SquashedDescription::from_args(args),
+        args.revision.is_none() && args.from.is_empty() && args.into.is_none(),
         &args.paths,
     )?;
     tx.finish(ui, tx_description)?;
     Ok(())
 }
 
+// TODO(#2882): Remove public visibility once `jj move` is deleted.
+pub(crate) enum SquashedDescription {
+    // Use this exact description.
+    Exact(String),
+    // Use the destination's description and discard the descriptions of the
+    // source revisions.
+    UseDestination,
+    // Combine the descriptions of the source and destination revisions.
+    Combine,
+}
+
+// TODO(#2882): Remove public visibility once `jj move` is deleted.
+impl SquashedDescription {
+    pub(crate) fn from_args(args: &SquashArgs) -> Self {
+        // These options are incompatible and Clap is configured to prevent this.
+        assert!(args.message_paragraphs.is_empty() || !args.use_destination_message);
+
+        if !args.message_paragraphs.is_empty() {
+            let desc = join_message_paragraphs(&args.message_paragraphs);
+            SquashedDescription::Exact(desc)
+        } else if args.use_destination_message {
+            SquashedDescription::UseDestination
+        } else {
+            SquashedDescription::Combine
+        }
+    }
+}
+
 #[allow(clippy::too_many_arguments)]
 pub fn move_diff(
     ui: &mut Ui,
@@ -140,15 +176,20 @@ pub fn move_diff(
     destination: &Commit,
     matcher: &dyn Matcher,
     diff_selector: &DiffSelector,
-    description: Option<String>,
+    description: SquashedDescription,
     no_rev_arg: bool,
     path_arg: &[String],
 ) -> Result<(), CommandError> {
     tx.base_workspace_helper()
-        .check_rewritable(sources.iter().chain(std::iter::once(destination)))?;
-    // Tree diffs to apply to the destination
-    let mut tree_diffs = vec![];
-    let mut abandoned_commits = vec![];
+        .check_rewritable(sources.iter().chain(std::iter::once(destination)).ids())?;
+
+    struct SourceCommit<'a> {
+        commit: &'a Commit,
+        parent_tree: MergedTree,
+        selected_tree: MergedTree,
+        abandon: bool,
+    }
+    let mut source_commits = vec![];
     for source in sources {
         let parent_tree = merge_commit_trees(tx.repo(), &source.parents())?;
         let source_tree = source.tree()?;
@@ -168,29 +209,26 @@ from the source will be moved into the destination.
             tx.format_commit_summary(source),
             tx.format_commit_summary(destination)
         );
-        let new_parent_tree_id =
+        let selected_tree_id =
             diff_selector.select(&parent_tree, &source_tree, matcher, Some(&instructions))?;
-        let new_parent_tree = tx.repo().store().get_root_tree(&new_parent_tree_id)?;
+        let selected_tree = tx.repo().store().get_root_tree(&selected_tree_id)?;
+        let abandon = selected_tree.id() == source_tree.id();
+        if !abandon && selected_tree_id == parent_tree.id() {
+            // Nothing selected from this commit. If it's abandoned (i.e. already empty), we
+            // still include it so `jj squash` can be used for abandoning an empty commit in
+            // the middle of a stack.
+            continue;
+        }
         // TODO: Do we want to optimize the case of moving to the parent commit (`jj
         // squash -r`)? The source tree will be unchanged in that case.
-
-        // Apply the reverse of the selected changes onto the source
-        let new_source_tree = source_tree.merge(&new_parent_tree, &parent_tree)?;
-        let abandon_source = new_source_tree.id() == parent_tree.id();
-        if abandon_source {
-            abandoned_commits.push(source);
-            tx.mut_repo().record_abandoned_commit(source.id().clone());
-        } else {
-            tx.mut_repo()
-                .rewrite_commit(settings, source)
-                .set_tree_id(new_source_tree.id().clone())
-                .write()?;
-        }
-        if new_parent_tree_id != parent_tree.id() {
-            tree_diffs.push((parent_tree, new_parent_tree));
-        }
+        source_commits.push(SourceCommit {
+            commit: source,
+            parent_tree,
+            selected_tree,
+            abandon,
+        });
     }
-    if tree_diffs.is_empty() {
+    if source_commits.is_empty() {
         if diff_selector.is_interactive() {
             return Err(user_error("No changes selected"));
         }
@@ -210,7 +248,25 @@ from the source will be moved into the destination.
                 )?;
             }
         }
+
+        return Ok(());
     }
+
+    for source in &source_commits {
+        if source.abandon {
+            tx.mut_repo()
+                .record_abandoned_commit(source.commit.id().clone());
+        } else {
+            let source_tree = source.commit.tree()?;
+            // Apply the reverse of the selected changes onto the source
+            let new_source_tree = source_tree.merge(&source.selected_tree, &source.parent_tree)?;
+            tx.mut_repo()
+                .rewrite_commit(settings, source.commit)
+                .set_tree_id(new_source_tree.id().clone())
+                .write()?;
+        }
+    }
+
     let mut rewritten_destination = destination.clone();
     if sources
         .iter()
@@ -226,15 +282,26 @@ from the source will be moved into the destination.
     }
     // Apply the selected changes onto the destination
     let mut destination_tree = rewritten_destination.tree()?;
-    for (tree1, tree2) in tree_diffs {
-        destination_tree = destination_tree.merge(&tree1, &tree2)?;
+    for source in &source_commits {
+        destination_tree = destination_tree.merge(&source.parent_tree, &source.selected_tree)?;
     }
     let description = match description {
-        Some(description) => description,
-        None => combine_messages(tx.base_repo(), &abandoned_commits, destination, settings)?,
+        SquashedDescription::Exact(description) => description,
+        SquashedDescription::UseDestination => destination.description().to_owned(),
+        SquashedDescription::Combine => {
+            let abandoned_commits = source_commits
+                .iter()
+                .filter_map(|source| source.abandon.then_some(source.commit))
+                .collect_vec();
+            combine_messages(tx.base_repo(), &abandoned_commits, destination, settings)?
+        }
     };
     let mut predecessors = vec![destination.id().clone()];
-    predecessors.extend(sources.iter().map(|source| source.id().clone()));
+    predecessors.extend(
+        source_commits
+            .iter()
+            .map(|source| source.commit.id().clone()),
+    );
     tx.mut_repo()
         .rewrite_commit(settings, &rewritten_destination)
         .set_tree_id(destination_tree.id().clone())
