@@ -25,10 +25,6 @@ use itertools::Itertools;
 
 // Lets the caller label strings and translates the labels to colors
 pub trait Formatter: Write {
-    fn write_str(&mut self, text: &str) -> io::Result<()> {
-        self.write_all(text.as_bytes())
-    }
-
     /// Returns the backing `Write`. This is useful for writing data that is
     /// already formatted, such as in the graphical log.
     fn raw(&mut self) -> &mut dyn Write;
@@ -77,9 +73,50 @@ where
     S: AsRef<str>,
 {
     pub fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
+        self.with_labeled(|formatter| formatter.write_fmt(args))
+    }
+
+    fn with_labeled(
+        &mut self,
+        write_inner: impl FnOnce(&mut dyn Formatter) -> io::Result<()>,
+    ) -> io::Result<()> {
         self.formatter
             .borrow_mut()
-            .with_label(self.label.as_ref(), |formatter| formatter.write_fmt(args))
+            .with_label(self.label.as_ref(), write_inner)
+    }
+}
+
+/// Like `LabeledWriter`, but also prints the `heading` once.
+///
+/// The `heading` will be printed within the first `write!()` or `writeln!()`
+/// invocation, which is handy because `io::Error` can be handled there.
+pub struct HeadingLabeledWriter<T, S, H> {
+    writer: LabeledWriter<T, S>,
+    heading: Option<H>,
+}
+
+impl<T, S, H> HeadingLabeledWriter<T, S, H> {
+    pub fn new(formatter: T, label: S, heading: H) -> Self {
+        HeadingLabeledWriter {
+            writer: LabeledWriter::new(formatter, label),
+            heading: Some(heading),
+        }
+    }
+}
+
+impl<'a, T, S, H> HeadingLabeledWriter<T, S, H>
+where
+    T: BorrowMut<dyn Formatter + 'a>,
+    S: AsRef<str>,
+    H: fmt::Display,
+{
+    pub fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
+        self.writer.with_labeled(|formatter| {
+            if let Some(heading) = self.heading.take() {
+                write!(formatter.labeled("heading"), "{heading}")?;
+            }
+            formatter.write_fmt(args)
+        })
     }
 }
 
@@ -333,7 +370,7 @@ fn rules_from_config(config: &config::Config) -> Result<Rules, config::ConfigErr
         match value.kind {
             config::ValueKind::String(color_name) => {
                 let style = Style {
-                    fg_color: Some(color_for_name(&color_name)?),
+                    fg_color: Some(color_for_name_or_hex(&color_name)?),
                     bg_color: None,
                     bold: None,
                     underlined: None,
@@ -344,12 +381,12 @@ fn rules_from_config(config: &config::Config) -> Result<Rules, config::ConfigErr
                 let mut style = Style::default();
                 if let Some(value) = style_table.get("fg") {
                     if let config::ValueKind::String(color_name) = &value.kind {
-                        style.fg_color = Some(color_for_name(color_name)?);
+                        style.fg_color = Some(color_for_name_or_hex(color_name)?);
                     }
                 }
                 if let Some(value) = style_table.get("bg") {
                     if let config::ValueKind::String(color_name) = &value.kind {
-                        style.bg_color = Some(color_for_name(color_name)?);
+                        style.bg_color = Some(color_for_name_or_hex(color_name)?);
                     }
                 }
                 if let Some(value) = style_table.get("bold") {
@@ -370,8 +407,8 @@ fn rules_from_config(config: &config::Config) -> Result<Rules, config::ConfigErr
     Ok(result)
 }
 
-fn color_for_name(color_name: &str) -> Result<Color, config::ConfigError> {
-    match color_name {
+fn color_for_name_or_hex(name_or_hex: &str) -> Result<Color, config::ConfigError> {
+    match name_or_hex {
         "default" => Ok(Color::Reset),
         "black" => Ok(Color::Black),
         "red" => Ok(Color::DarkRed),
@@ -389,9 +426,25 @@ fn color_for_name(color_name: &str) -> Result<Color, config::ConfigError> {
         "bright magenta" => Ok(Color::Magenta),
         "bright cyan" => Ok(Color::Cyan),
         "bright white" => Ok(Color::White),
-        _ => Err(config::ConfigError::Message(format!(
-            "invalid color: {color_name}"
-        ))),
+        _ => color_for_hex(name_or_hex)
+            .ok_or_else(|| config::ConfigError::Message(format!("invalid color: {}", name_or_hex))),
+    }
+}
+
+fn color_for_hex(color: &str) -> Option<Color> {
+    if color.len() == 7
+        && color.starts_with('#')
+        && color[1..].chars().all(|c| c.is_ascii_hexdigit())
+    {
+        let r = u8::from_str_radix(&color[1..3], 16);
+        let g = u8::from_str_radix(&color[3..5], 16);
+        let b = u8::from_str_radix(&color[5..7], 16);
+        match (r, g, b) {
+            (Ok(r), Ok(g), Ok(b)) => Some(Color::Rgb { r, g, b }),
+            _ => None,
+        }
+    } else {
+        None
     }
 }
 
@@ -476,7 +529,7 @@ impl<W: Write> Drop for ColorFormatter<W> {
 /// the destination formatter has already been labeled, the recorded labels
 /// will be stacked on top of the existing labels, and the subsequent data
 /// may be colorized differently.
-#[derive(Debug, Default)]
+#[derive(Clone, Debug, Default)]
 pub struct FormatRecorder {
     data: Vec<u8>,
     label_ops: Vec<(usize, LabelOp)>,
@@ -593,7 +646,7 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = PlainTextFormatter::new(&mut output);
         formatter.push_label("warning").unwrap();
-        formatter.write_str("hello").unwrap();
+        write!(formatter, "hello").unwrap();
         formatter.pop_label().unwrap();
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"hello");
     }
@@ -603,7 +656,7 @@ mod tests {
         // Test that ANSI codes in the input text are NOT escaped.
         let mut output: Vec<u8> = vec![];
         let mut formatter = PlainTextFormatter::new(&mut output);
-        formatter.write_str("\x1b[1mactually bold\x1b[0m").unwrap();
+        write!(formatter, "\x1b[1mactually bold\x1b[0m").unwrap();
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"[1mactually bold[0m");
     }
 
@@ -612,9 +665,7 @@ mod tests {
         // Test that ANSI codes in the input text are escaped.
         let mut output: Vec<u8> = vec![];
         let mut formatter = SanitizingFormatter::new(&mut output);
-        formatter
-            .write_str("\x1b[1mnot actually bold\x1b[0m")
-            .unwrap();
+        write!(formatter, "\x1b[1mnot actually bold\x1b[0m").unwrap();
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"␛[1mnot actually bold␛[0m");
     }
 
@@ -651,9 +702,9 @@ mod tests {
             ColorFormatter::for_config(&mut output, &config_builder.build().unwrap()).unwrap();
         for color in colors {
             formatter.push_label(&color.replace(' ', "-")).unwrap();
-            formatter.write_str(&format!(" {color} ")).unwrap();
+            write!(formatter, " {color} ").unwrap();
             formatter.pop_label().unwrap();
-            formatter.write_str("\n").unwrap();
+            writeln!(formatter).unwrap();
         }
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @r###"
@@ -676,6 +727,38 @@ mod tests {
         "###);
     }
 
+    #[test]
+    fn test_color_formatter_hex_colors() {
+        // Test the color code for each color.
+        let labels_and_colors = [
+            ["black", "#000000"],
+            ["white", "#ffffff"],
+            ["pastel-blue", "#AFE0D9"],
+        ];
+        let mut config_builder = config::Config::builder();
+        for [label, color] in labels_and_colors {
+            // Use the color name as the label.
+            config_builder = config_builder
+                .set_override(format!("colors.{}", label), color)
+                .unwrap();
+        }
+        let mut output: Vec<u8> = vec![];
+        let mut formatter =
+            ColorFormatter::for_config(&mut output, &config_builder.build().unwrap()).unwrap();
+        for [label, _] in labels_and_colors {
+            formatter.push_label(&label.replace(' ', "-")).unwrap();
+            write!(formatter, " {label} ").unwrap();
+            formatter.pop_label().unwrap();
+            writeln!(formatter).unwrap();
+        }
+        drop(formatter);
+        insta::assert_snapshot!(String::from_utf8(output).unwrap(), @r###"
+        [38;2;0;0;0m black [39m
+        [38;2;255;255;255m white [39m
+        [38;2;175;224;217m pastel-blue [39m
+        "###);
+    }
+
     #[test]
     fn test_color_formatter_single_label() {
         // Test that a single label can be colored and that the color is reset
@@ -687,11 +770,11 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
-        formatter.write_str(" before ").unwrap();
+        write!(formatter, " before ").unwrap();
         formatter.push_label("inside").unwrap();
-        formatter.write_str(" inside ").unwrap();
+        write!(formatter, " inside ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" after ").unwrap();
+        write!(formatter, " after ").unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @" before [38;5;2m inside [39m after ");
     }
@@ -712,31 +795,31 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("red_fg").unwrap();
-        formatter.write_str(" fg only ").unwrap();
+        write!(formatter, " fg only ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("\n").unwrap();
+        writeln!(formatter).unwrap();
         formatter.push_label("blue_bg").unwrap();
-        formatter.write_str(" bg only ").unwrap();
+        write!(formatter, " bg only ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("\n").unwrap();
+        writeln!(formatter).unwrap();
         formatter.push_label("bold_font").unwrap();
-        formatter.write_str(" bold only ").unwrap();
+        write!(formatter, " bold only ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("\n").unwrap();
+        writeln!(formatter).unwrap();
         formatter.push_label("underlined_text").unwrap();
-        formatter.write_str(" underlined only ").unwrap();
+        write!(formatter, " underlined only ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("\n").unwrap();
+        writeln!(formatter).unwrap();
         formatter.push_label("multiple").unwrap();
-        formatter.write_str(" single rule ").unwrap();
+        write!(formatter, " single rule ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("\n").unwrap();
+        writeln!(formatter).unwrap();
         formatter.push_label("red_fg").unwrap();
         formatter.push_label("blue_bg").unwrap();
-        formatter.write_str(" two rules ").unwrap();
+        write!(formatter, " two rules ").unwrap();
         formatter.pop_label().unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("\n").unwrap();
+        writeln!(formatter).unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @r###"
         [38;5;1m fg only [39m
@@ -760,11 +843,11 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("not_bold").unwrap();
-        formatter.write_str(" not bold ").unwrap();
+        write!(formatter, " not bold ").unwrap();
         formatter.push_label("bold_font").unwrap();
-        formatter.write_str(" bold ").unwrap();
+        write!(formatter, " bold ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" not bold again ").unwrap();
+        write!(formatter, " not bold again ").unwrap();
         formatter.pop_label().unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"[4m[38;5;1m[48;5;4m not bold [1m bold [0m[4m[38;5;1m[48;5;4m not bold again [24m[39m[49m");
@@ -781,14 +864,14 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
-        formatter.write_str("before").unwrap();
+        write!(formatter, "before").unwrap();
         formatter.push_label("red").unwrap();
-        formatter.write_str("first").unwrap();
+        write!(formatter, "first").unwrap();
         formatter.pop_label().unwrap();
         formatter.push_label("green").unwrap();
-        formatter.write_str("second").unwrap();
+        write!(formatter, "second").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str("after").unwrap();
+        write!(formatter, "after").unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"before[38;5;1mfirst[39m[38;5;2msecond[39mafter");
     }
@@ -804,9 +887,7 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("red").unwrap();
-        formatter
-            .write_str("\x1b[1mnot actually bold\x1b[0m")
-            .unwrap();
+        write!(formatter, "\x1b[1mnot actually bold\x1b[0m").unwrap();
         formatter.pop_label().unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"[38;5;1m␛[1mnot actually bold␛[0m[39m");
@@ -826,15 +907,15 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
-        formatter.write_str(" before outer ").unwrap();
+        write!(formatter, " before outer ").unwrap();
         formatter.push_label("outer").unwrap();
-        formatter.write_str(" before inner ").unwrap();
+        write!(formatter, " before inner ").unwrap();
         formatter.push_label("inner").unwrap();
-        formatter.write_str(" inside inner ").unwrap();
+        write!(formatter, " inside inner ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" after inner ").unwrap();
+        write!(formatter, " after inner ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" after outer ").unwrap();
+        write!(formatter, " after outer ").unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(),
         @" before outer [38;5;4m before inner [38;5;2m inside inner [38;5;4m after inner [39m after outer ");
@@ -851,11 +932,11 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("outer").unwrap();
-        formatter.write_str(" not colored ").unwrap();
+        write!(formatter, " not colored ").unwrap();
         formatter.push_label("inner").unwrap();
-        formatter.write_str(" colored ").unwrap();
+        write!(formatter, " colored ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" not colored ").unwrap();
+        write!(formatter, " not colored ").unwrap();
         formatter.pop_label().unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(),
@@ -879,6 +960,23 @@ mod tests {
         @"invalid color: bloo");
     }
 
+    #[test]
+    fn test_color_formatter_unrecognized_hex_color() {
+        // An unrecognized hex color causes an error.
+        let config = config_from_string(
+            r##"
+            colors."outer" = "red"
+            colors."outer inner" = "#ffgggg"
+            "##,
+        );
+        let mut output: Vec<u8> = vec![];
+        let err = ColorFormatter::for_config(&mut output, &config)
+            .unwrap_err()
+            .to_string();
+        insta::assert_snapshot!(err,
+            @"invalid color: #ffgggg");
+    }
+
     #[test]
     fn test_color_formatter_normal_color() {
         // The "default" color resets the color. It is possible to reset only the
@@ -893,15 +991,15 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("outer").unwrap();
-        formatter.write_str("Blue on yellow, ").unwrap();
+        write!(formatter, "Blue on yellow, ").unwrap();
         formatter.push_label("default_fg").unwrap();
-        formatter.write_str(" default fg, ").unwrap();
+        write!(formatter, " default fg, ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" and back.\nBlue on yellow, ").unwrap();
+        write!(formatter, " and back.\nBlue on yellow, ").unwrap();
         formatter.push_label("default_bg").unwrap();
-        formatter.write_str(" default bg, ").unwrap();
+        write!(formatter, " default bg, ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" and back.").unwrap();
+        write!(formatter, " and back.").unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(),
         @r###"
@@ -923,7 +1021,7 @@ mod tests {
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("outer1").unwrap();
         formatter.push_label("inner2").unwrap();
-        formatter.write_str(" hello ").unwrap();
+        write!(formatter, " hello ").unwrap();
         formatter.pop_label().unwrap();
         formatter.pop_label().unwrap();
         drop(formatter);
@@ -943,7 +1041,7 @@ mod tests {
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("outer").unwrap();
         formatter.push_label("inner").unwrap();
-        formatter.write_str(" hello ").unwrap();
+        write!(formatter, " hello ").unwrap();
         formatter.pop_label().unwrap();
         formatter.pop_label().unwrap();
         drop(formatter);
@@ -964,15 +1062,15 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("a").unwrap();
-        formatter.write_str(" a1 ").unwrap();
+        write!(formatter, " a1 ").unwrap();
         formatter.push_label("b").unwrap();
-        formatter.write_str(" b1 ").unwrap();
+        write!(formatter, " b1 ").unwrap();
         formatter.push_label("c").unwrap();
-        formatter.write_str(" c ").unwrap();
+        write!(formatter, " c ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" b2 ").unwrap();
+        write!(formatter, " b2 ").unwrap();
         formatter.pop_label().unwrap();
-        formatter.write_str(" a2 ").unwrap();
+        write!(formatter, " a2 ").unwrap();
         formatter.pop_label().unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(),
@@ -992,20 +1090,54 @@ mod tests {
         let mut formatter = ColorFormatter::for_config(&mut output, &config).unwrap();
         formatter.push_label("outer").unwrap();
         formatter.push_label("inner").unwrap();
-        formatter.write_str(" inside ").unwrap();
+        write!(formatter, " inside ").unwrap();
         drop(formatter);
         insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"[38;5;2m inside [39m");
     }
 
+    #[test]
+    fn test_heading_labeled_writer() {
+        let config = config_from_string(
+            r#"
+        colors.inner = "green"
+        colors."inner heading" = "red"
+        "#,
+        );
+        let mut output: Vec<u8> = vec![];
+        let mut formatter: Box<dyn Formatter> =
+            Box::new(ColorFormatter::for_config(&mut output, &config).unwrap());
+        HeadingLabeledWriter::new(formatter.as_mut(), "inner", "Should be noop: ");
+        let mut writer = HeadingLabeledWriter::new(formatter.as_mut(), "inner", "Heading: ");
+        write!(writer, "Message").unwrap();
+        writeln!(writer, " continues").unwrap();
+        drop(formatter);
+        insta::assert_snapshot!(String::from_utf8(output).unwrap(), @r###"
+        [38;5;1mHeading: [38;5;2mMessage[39m[38;5;2m continues[39m
+        "###);
+    }
+
+    #[test]
+    fn test_heading_labeled_writer_empty_string() {
+        let mut output: Vec<u8> = vec![];
+        let mut formatter: Box<dyn Formatter> = Box::new(PlainTextFormatter::new(&mut output));
+        let mut writer = HeadingLabeledWriter::new(formatter.as_mut(), "inner", "Heading: ");
+        // write_fmt() is called even if the format string is empty. I don't
+        // know if that's guaranteed, but let's record the current behavior.
+        write!(writer, "").unwrap();
+        write!(writer, "").unwrap();
+        drop(formatter);
+        insta::assert_snapshot!(String::from_utf8(output).unwrap(), @"Heading: ");
+    }
+
     #[test]
     fn test_format_recorder() {
         let mut recorder = FormatRecorder::new();
-        recorder.write_str(" outer1 ").unwrap();
+        write!(recorder, " outer1 ").unwrap();
         recorder.push_label("inner").unwrap();
-        recorder.write_str(" inner1 ").unwrap();
-        recorder.write_str(" inner2 ").unwrap();
+        write!(recorder, " inner1 ").unwrap();
+        write!(recorder, " inner2 ").unwrap();
         recorder.pop_label().unwrap();
-        recorder.write_str(" outer2 ").unwrap();
+        write!(recorder, " outer2 ").unwrap();
 
         insta::assert_snapshot!(
             str::from_utf8(recorder.data()).unwrap(),
