@@ -51,14 +51,16 @@ impl OperationTemplateLanguage {
     pub fn new(
         root_op_id: &OperationId,
         current_op_id: Option<&OperationId>,
-        extension: Option<&dyn OperationTemplateLanguageExtension>,
+        extensions: &[impl AsRef<dyn OperationTemplateLanguageExtension>],
     ) -> Self {
         let mut build_fn_table = OperationTemplateBuildFnTable::builtin();
         let mut cache_extensions = ExtensionsMap::empty();
-        if let Some(extension) = extension {
-            let ext_table = extension.build_fn_table();
-            build_fn_table.merge(ext_table);
-            extension.build_cache_extensions(&mut cache_extensions);
+
+        for extension in extensions {
+            build_fn_table.merge(extension.as_ref().build_fn_table());
+            extension
+                .as_ref()
+                .build_cache_extensions(&mut cache_extensions);
         }
 
         OperationTemplateLanguage {
@@ -90,6 +92,7 @@ impl TemplateLanguage<'static> for OperationTemplateLanguage {
         property: Self::Property,
         function: &FunctionCallNode,
     ) -> TemplateParseResult<Self::Property> {
+        let type_name = property.type_name();
         match property {
             OperationTemplatePropertyKind::Core(property) => {
                 let table = &self.build_fn_table.core;
@@ -97,12 +100,12 @@ impl TemplateLanguage<'static> for OperationTemplateLanguage {
             }
             OperationTemplatePropertyKind::Operation(property) => {
                 let table = &self.build_fn_table.operation_methods;
-                let build = template_parser::lookup_method("Operation", table, function)?;
+                let build = template_parser::lookup_method(type_name, table, function)?;
                 build(self, build_ctx, property, function)
             }
             OperationTemplatePropertyKind::OperationId(property) => {
                 let table = &self.build_fn_table.operation_id_methods;
-                let build = template_parser::lookup_method("OperationId", table, function)?;
+                let build = template_parser::lookup_method(type_name, table, function)?;
                 build(self, build_ctx, property, function)
             }
         }
@@ -134,6 +137,14 @@ pub enum OperationTemplatePropertyKind {
 }
 
 impl IntoTemplateProperty<'static> for OperationTemplatePropertyKind {
+    fn type_name(&self) -> &'static str {
+        match self {
+            OperationTemplatePropertyKind::Core(property) => property.type_name(),
+            OperationTemplatePropertyKind::Operation(_) => "Operation",
+            OperationTemplatePropertyKind::OperationId(_) => "OperationId",
+        }
+    }
+
     fn try_into_boolean(self) -> Option<Box<dyn TemplateProperty<Output = bool>>> {
         match self {
             OperationTemplatePropertyKind::Core(property) => property.try_into_boolean(),
@@ -249,6 +260,14 @@ fn builtin_operation_methods() -> OperationTemplateBuildMethodFnMap<Operation> {
         });
         Ok(L::wrap_string(out_property))
     });
+    map.insert(
+        "snapshot",
+        |_language, _build_ctx, self_property, function| {
+            template_parser::expect_no_arguments(function)?;
+            let out_property = self_property.map(|op| op.metadata().is_snapshot);
+            Ok(L::wrap_boolean(out_property))
+        },
+    );
     map.insert("time", |_language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
         let out_property = self_property.map(|op| TimestampRange {
