@@ -16,6 +16,7 @@ use std::collections::HashMap;
 use std::{error, fmt, mem};
 
 use itertools::Itertools as _;
+use jj_lib::dsl_util::{collect_similar, StringLiteralParser};
 use once_cell::sync::Lazy;
 use pest::iterators::{Pair, Pairs};
 use pest::pratt_parser::{Assoc, Op, PrattParser};
@@ -27,15 +28,22 @@ use thiserror::Error;
 #[grammar = "template.pest"]
 struct TemplateParser;
 
+const STRING_LITERAL_PARSER: StringLiteralParser<Rule> = StringLiteralParser {
+    content_rule: Rule::string_content,
+    escape_rule: Rule::string_escape,
+};
+
 impl Rule {
     fn to_symbol(self) -> Option<&'static str> {
         match self {
             Rule::EOI => None,
             Rule::whitespace => None,
-            Rule::escape => None,
-            Rule::literal_char => None,
-            Rule::raw_literal => None,
-            Rule::literal => None,
+            Rule::string_escape => None,
+            Rule::string_content_char => None,
+            Rule::string_content => None,
+            Rule::string_literal => None,
+            Rule::raw_string_content => None,
+            Rule::raw_string_literal => None,
             Rule::integer_literal => None,
             Rule::identifier => None,
             Rule::concat_op => Some("++"),
@@ -52,7 +60,6 @@ impl Rule {
             Rule::primary => None,
             Rule::term => None,
             Rule::expression => None,
-            Rule::concat => None,
             Rule::template => None,
             Rule::program => None,
             Rule::function_alias_declaration => None,
@@ -147,8 +154,9 @@ impl TemplateParseError {
         )
     }
 
-    pub fn expected_type(type_name: &str, span: pest::Span<'_>) -> Self {
-        let message = format!(r#"Expected expression of type "{type_name}""#);
+    pub fn expected_type(expected: &str, actual: &str, span: pest::Span<'_>) -> Self {
+        let message =
+            format!(r#"Expected expression of type "{expected}", but actual type is "{actual}""#);
         TemplateParseError::expression(message, span)
     }
 
@@ -315,29 +323,6 @@ fn parse_identifier_name(pair: Pair<Rule>) -> TemplateParseResult<&str> {
     }
 }
 
-fn parse_string_literal(pair: Pair<Rule>) -> String {
-    assert_eq!(pair.as_rule(), Rule::literal);
-    let mut result = String::new();
-    for part in pair.into_inner() {
-        match part.as_rule() {
-            Rule::raw_literal => {
-                result.push_str(part.as_str());
-            }
-            Rule::escape => match &part.as_str()[1..] {
-                "\"" => result.push('"'),
-                "\\" => result.push('\\'),
-                "t" => result.push('\t'),
-                "r" => result.push('\r'),
-                "n" => result.push('\n'),
-                "0" => result.push('\0'),
-                char => panic!("invalid escape: \\{char:?}"),
-            },
-            _ => panic!("unexpected part of string: {part:?}"),
-        }
-    }
-    result
-}
-
 fn parse_formal_parameters(params_pair: Pair<Rule>) -> TemplateParseResult<Vec<&str>> {
     assert_eq!(params_pair.as_rule(), Rule::formal_parameters);
     let params_span = params_pair.as_span();
@@ -397,8 +382,14 @@ fn parse_term_node(pair: Pair<Rule>) -> TemplateParseResult<ExpressionNode> {
     let expr = inner.next().unwrap();
     let span = expr.as_span();
     let primary = match expr.as_rule() {
-        Rule::literal => {
-            let text = parse_string_literal(expr);
+        Rule::string_literal => {
+            let text = STRING_LITERAL_PARSER.parse(expr.into_inner());
+            ExpressionNode::new(ExpressionKind::String(text), span)
+        }
+        Rule::raw_string_literal => {
+            let (content,) = expr.into_inner().collect_tuple().unwrap();
+            assert_eq!(content.as_rule(), Rule::raw_string_content);
+            let text = content.as_str().to_owned();
             ExpressionNode::new(ExpressionKind::String(text), span)
         }
         Rule::integer_literal => {
@@ -476,7 +467,6 @@ fn parse_template_node(pair: Pair<Rule>) -> TemplateParseResult<ExpressionNode>
     let mut nodes: Vec<_> = inner
         .filter_map(|pair| match pair.as_rule() {
             Rule::concat_op => None,
-            Rule::term => Some(parse_term_node(pair)),
             Rule::expression => Some(parse_expression_node(pair)),
             r => panic!("unexpected template item rule {r:?}"),
         })
@@ -906,23 +896,6 @@ pub fn lookup_method<'a, V>(
     }
 }
 
-// TODO: merge with revset::collect_similar()?
-fn collect_similar<I>(name: &str, candidates: I) -> Vec<String>
-where
-    I: IntoIterator,
-    I::Item: AsRef<str>,
-{
-    candidates
-        .into_iter()
-        .filter(|cand| {
-            // The parameter is borrowed from clap f5540d26
-            strsim::jaro(name, cand.as_ref()) > 0.7
-        })
-        .map(|s| s.as_ref().to_owned())
-        .sorted_unstable()
-        .collect()
-}
-
 #[cfg(test)]
 mod tests {
     use assert_matches::assert_matches;
@@ -1070,11 +1043,16 @@ mod tests {
             parse_normalized("x || (y && (z.h()))").unwrap(),
         );
 
-        // Top-level expression is allowed, but not in concatenation
-        assert!(parse_template(r"x && y").is_ok());
-        assert!(parse_template(r"f(x && y)").is_ok());
-        assert!(parse_template(r"x && y ++ z").is_err());
-        assert!(parse_template(r"(x && y) ++ z").is_ok());
+        // Logical operator bounds more tightly than concatenation. This might
+        // not be so intuitive, but should be harmless.
+        assert_eq!(
+            parse_normalized(r"x && y ++ z").unwrap(),
+            parse_normalized(r"(x && y) ++ z").unwrap(),
+        );
+        assert_eq!(
+            parse_normalized(r"x ++ y || z").unwrap(),
+            parse_normalized(r"x ++ (y || z)").unwrap(),
+        );
 
         // Expression span
         assert_eq!(parse_template(" ! x ").unwrap().span.as_str(), "! x");
@@ -1209,6 +1187,24 @@ mod tests {
             parse_into_kind(r#" "\y" "#),
             Err(TemplateParseErrorKind::SyntaxError),
         );
+
+        // Single-quoted raw string
+        assert_eq!(
+            parse_into_kind(r#" '' "#),
+            Ok(ExpressionKind::String("".to_owned())),
+        );
+        assert_eq!(
+            parse_into_kind(r#" 'a\n' "#),
+            Ok(ExpressionKind::String(r"a\n".to_owned())),
+        );
+        assert_eq!(
+            parse_into_kind(r#" '\' "#),
+            Ok(ExpressionKind::String(r"\".to_owned())),
+        );
+        assert_eq!(
+            parse_into_kind(r#" '"' "#),
+            Ok(ExpressionKind::String(r#"""#.to_owned())),
+        );
     }
 
     #[test]
