@@ -121,14 +121,14 @@ fn test_new_merge() {
     // `jj merge` with less than two arguments is an error
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["merge"]);
     insta::assert_snapshot!(stderr, @r###"
-    warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent
-    warning: `jj merge` will be removed in a future version, and this will be a hard error
+    Warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent
+    Warning: `jj merge` will be removed in a future version, and this will be a hard error
     Error: Merge requires at least two revisions
     "###);
     let stderr = test_env.jj_cmd_cli_error(&repo_path, &["merge", "main"]);
     insta::assert_snapshot!(stderr, @r###"
-    warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent
-    warning: `jj merge` will be removed in a future version, and this will be a hard error
+    Warning: `jj merge` is deprecated; use `jj new` instead, which is equivalent
+    Warning: `jj merge` will be removed in a future version, and this will be a hard error
     Error: Merge requires at least two revisions
     "###);
 
@@ -137,11 +137,18 @@ fn test_new_merge() {
     insta::assert_snapshot!(stderr, @r###"
     Error: More than one revset resolved to revision 3a44e52b073c
     "###);
+    // if prefixed with all:, duplicates are allowed
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["new", "@", "all:visible_heads()"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: xznxytkn dddeb489 (empty) (no description set)
+    Parent commit      : wqnwkozp 3a44e52b (empty) (no description set)
+    "###);
 
     // merge with root
     let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "@", "root()"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Cannot merge with root revision
+    Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
     "###);
 }
 
@@ -417,26 +424,10 @@ fn test_new_insert_before_no_root_merge() {
     ◉  root
     "###);
 
-    let (stdout, stderr) =
-        test_env.jj_cmd_ok(&repo_path, &["new", "--insert-before", "-m", "G", "B", "D"]);
-    insta::assert_snapshot!(stdout, @"");
+    let stderr =
+        test_env.jj_cmd_failure(&repo_path, &["new", "--insert-before", "-m", "G", "B", "D"]);
     insta::assert_snapshot!(stderr, @r###"
-    Rebased 4 descendant commits
-    Working copy now at: kxryzmor bf9fc493 (empty) G
-    Parent commit      : qpvuntsm 65b1ef43 A | (empty) A
-    "###);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r###"
-    ◉    F
-    ├─╮
-    │ ◉  E
-    ◉ │  D
-    │ │ ◉  C
-    │ │ ◉  B
-    ├───╯
-    @ │  G
-    ◉ │  A
-    ├─╯
-    ◉  root
+    Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
     "###);
 }
 
@@ -495,10 +486,9 @@ fn test_new_conflicting_branches() {
     Error: Revset "foo" resolved to more than one revision
     Hint: Branch foo resolved to multiple revisions because it's conflicted.
     It resolved to these revisions:
-    kkmpptxz 66c6502d foo?? | (empty) two
-    qpvuntsm a9330854 foo?? | (empty) one
-    Set which revision the branch points to with `jj branch set foo -r <REVISION>`.
-    Prefix the expression with 'all' to allow any number of revisions (i.e. 'all:foo').
+      kkmpptxz 66c6502d foo?? | (empty) two
+      qpvuntsm a9330854 foo?? | (empty) one
+    Hint: Set which revision the branch points to with `jj branch set foo -r <REVISION>`.
     "###);
 }
 
@@ -518,10 +508,24 @@ fn test_new_conflicting_change_ids() {
     insta::assert_snapshot!(stderr, @r###"
     Error: Revset "qpvuntsm" resolved to more than one revision
     Hint: The revset "qpvuntsm" resolved to these revisions:
-    qpvuntsm?? d2ae6806 (empty) two
-    qpvuntsm?? a9330854 (empty) one
-    Some of these commits have the same change id. Abandon one of them with `jj abandon -r <REVISION>`.
-    Prefix the expression with 'all' to allow any number of revisions (i.e. 'all:qpvuntsm').
+      qpvuntsm?? d2ae6806 (empty) two
+      qpvuntsm?? a9330854 (empty) one
+    Hint: Some of these commits have the same change id. Abandon one of them with `jj abandon -r <REVISION>`.
+    "###);
+}
+
+#[test]
+fn test_new_error_revision_does_not_exist() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.jj_cmd_ok(&repo_path, &["describe", "-m", "one"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-m", "two"]);
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "this"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Revision "this" doesn't exist
     "###);
 }
 
