@@ -136,13 +136,16 @@ fn test_op_log_with_no_template() {
 
     For more information, try '--help'.
     Hint: The following template aliases are defined:
-    - builtin_change_id_with_hidden_and_divergent_info
     - builtin_log_comfortable
     - builtin_log_compact
     - builtin_log_detailed
+    - builtin_log_node
+    - builtin_log_node_ascii
     - builtin_log_oneline
     - builtin_op_log_comfortable
     - builtin_op_log_compact
+    - builtin_op_log_node
+    - builtin_op_log_node_ascii
     - commit_summary_separator
     - description_placeholder
     - email_placeholder
