@@ -111,7 +111,7 @@ fn test_templater_parse_error() {
       | ^-----^
       |
       = Keyword "builtin" doesn't exist
-    Hint: Did you mean "builtin_change_id_with_hidden_and_divergent_info", "builtin_log_comfortable", "builtin_log_compact", "builtin_log_detailed", "builtin_log_oneline", "builtin_op_log_comfortable", "builtin_op_log_compact"?
+    Hint: Did you mean "builtin_log_comfortable", "builtin_log_compact", "builtin_log_detailed", "builtin_log_node", "builtin_log_node_ascii", "builtin_log_oneline", "builtin_op_log_comfortable", "builtin_op_log_compact", "builtin_op_log_node", "builtin_op_log_node_ascii"?
     "###);
 }
 
@@ -124,7 +124,7 @@ fn test_templater_upper_lower() {
 
     insta::assert_snapshot!(
       render(r#"change_id.shortest(4).upper() ++ change_id.shortest(4).upper().lower()"#),
-      @"[1m[38;5;5mZ[0m[38;5;8mZZZ[39m[1m[38;5;5mz[0m[38;5;8mzzz[39m");
+      @"[1m[38;5;5mZ[0m[38;5;8mZZZ[1m[38;5;5mz[0m[38;5;8mzzz[39m");
     insta::assert_snapshot!(
       render(r#""Hello".upper() ++ "Hello".lower()"#), @"HELLOhello");
 }
@@ -208,7 +208,7 @@ fn test_templater_alias() {
     1 | identity(identity(commit_id.short("")))
       |                                   ^^
       |
-      = Expected expression of type "Integer"
+      = Expected expression of type "Integer", but actual type is "String"
     "###);
 
     insta::assert_snapshot!(render_err("commit_id ++ recurse"), @r###"
@@ -273,7 +273,7 @@ fn test_templater_alias() {
     1 | coalesce(label("x", "not boolean"), "")
       |          ^-----------------------^
       |
-      = Expected expression of type "Boolean"
+      = Expected expression of type "Boolean", but actual type is "Template"
     "###);
 }
 
