@@ -615,6 +615,8 @@ impl TreeState {
         Ok(())
     }
 
+    #[allow(unknown_lints)] // XXX FIXME (aseipp): nightly bogons; re-test this occasionally
+    #[allow(clippy::assigning_clones)]
     fn save(&mut self) -> Result<(), TreeStateError> {
         let mut proto: crate::protos::working_copy::TreeState = Default::default();
         match &self.tree_id {
@@ -1043,7 +1045,7 @@ impl TreeState {
                 let repo_paths = trace_span!("processing fsmonitor paths").in_scope(|| {
                     changed_files
                         .into_iter()
-                        .filter_map(RepoPathBuf::from_relative_path)
+                        .filter_map(|path| RepoPathBuf::from_relative_path(path).ok())
                         .collect_vec()
                 });
 
@@ -1457,9 +1459,10 @@ impl TreeState {
         Ok(())
     }
 
-    pub fn reset_to_empty(&mut self) {
+    pub async fn recover(&mut self, new_tree: &MergedTree) -> Result<(), ResetError> {
         self.file_states.clear();
         self.tree_id = self.store.empty_merged_tree_id();
+        self.reset(new_tree).await
     }
 }
 
@@ -1703,8 +1706,12 @@ impl WorkingCopyFactory for LocalWorkingCopyFactory {
         store: Arc<Store>,
         working_copy_path: PathBuf,
         state_path: PathBuf,
-    ) -> Box<dyn WorkingCopy> {
-        Box::new(LocalWorkingCopy::load(store, working_copy_path, state_path))
+    ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError> {
+        Ok(Box::new(LocalWorkingCopy::load(
+            store,
+            working_copy_path,
+            state_path,
+        )))
     }
 }
 
@@ -1778,14 +1785,16 @@ impl LockedWorkingCopy for LockedLocalWorkingCopy {
         Ok(())
     }
 
-    fn reset_to_empty(&mut self) -> Result<(), ResetError> {
+    fn recover(&mut self, commit: &Commit) -> Result<(), ResetError> {
+        let new_tree = commit.tree()?;
         self.wc
             .tree_state_mut()
             .map_err(|err| ResetError::Other {
                 message: "Failed to read the working copy state".to_string(),
                 err: err.into(),
             })?
-            .reset_to_empty();
+            .recover(&new_tree)
+            .block_on()?;
         self.tree_state_dirty = true;
         Ok(())
     }
