@@ -14,6 +14,7 @@
 
 #![allow(missing_docs)]
 
+use std::collections::hash_map::Entry;
 use std::collections::{HashMap, HashSet};
 use std::fmt::{Debug, Formatter};
 use std::io::ErrorKind;
@@ -49,7 +50,8 @@ use crate::operation::Operation;
 use crate::refs::{
     diff_named_ref_targets, diff_named_remote_refs, merge_ref_targets, merge_remote_refs,
 };
-use crate::rewrite::{DescendantRebaser, RebaseOptions};
+use crate::revset::{RevsetEvaluationError, RevsetExpression, RevsetIteratorExt};
+use crate::rewrite::{CommitRewriter, DescendantRebaser, RebaseOptions};
 use crate::settings::{RepoSettings, UserSettings};
 use crate::signing::{SignInitError, Signer};
 use crate::simple_op_heads_store::SimpleOpHeadsStore;
@@ -57,7 +59,6 @@ use crate::simple_op_store::SimpleOpStore;
 use crate::store::Store;
 use crate::submodule_store::SubmoduleStore;
 use crate::transaction::Transaction;
-use crate::tree::TreeMergeError;
 use crate::view::View;
 use crate::{backend, dag_walk, op_store, revset};
 
@@ -348,6 +349,19 @@ type IndexStoreFactory =
     Box<dyn Fn(&UserSettings, &Path) -> Result<Box<dyn IndexStore>, BackendLoadError>>;
 type SubmoduleStoreFactory = Box<dyn Fn(&UserSettings, &Path) -> Box<dyn SubmoduleStore>>;
 
+pub fn merge_factories_map<F>(base: &mut HashMap<String, F>, ext: HashMap<String, F>) {
+    for (name, factory) in ext {
+        match base.entry(name) {
+            Entry::Vacant(v) => {
+                v.insert(factory);
+            }
+            Entry::Occupied(o) => {
+                panic!("Conflicting factory definitions for '{}' factory", o.key())
+            }
+        }
+    }
+}
+
 pub struct StoreFactories {
     backend_factories: HashMap<String, BackendFactory>,
     op_store_factories: HashMap<String, OpStoreFactory>,
@@ -427,6 +441,25 @@ impl StoreFactories {
         }
     }
 
+    pub fn merge(&mut self, ext: StoreFactories) {
+        let StoreFactories {
+            backend_factories,
+            op_store_factories,
+            op_heads_store_factories,
+            index_store_factories,
+            submodule_store_factories,
+        } = ext;
+
+        merge_factories_map(&mut self.backend_factories, backend_factories);
+        merge_factories_map(&mut self.op_store_factories, op_store_factories);
+        merge_factories_map(&mut self.op_heads_store_factories, op_heads_store_factories);
+        merge_factories_map(&mut self.index_store_factories, index_store_factories);
+        merge_factories_map(
+            &mut self.submodule_store_factories,
+            submodule_store_factories,
+        );
+    }
+
     pub fn add_backend(&mut self, name: &str, factory: BackendFactory) {
         self.backend_factories.insert(name.to_string(), factory);
     }
@@ -580,7 +613,7 @@ pub fn read_store_type_compat(
 #[derive(Debug, Error)]
 pub enum RepoLoaderError {
     #[error(transparent)]
-    TreeMerge(#[from] TreeMergeError),
+    Backend(#[from] BackendError),
     #[error(transparent)]
     OpHeadResolution(#[from] OpHeadResolutionError),
     #[error(transparent)]
@@ -728,12 +761,26 @@ impl RepoLoader {
     }
 }
 
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+pub(crate) enum RewriteType {
+    Rewritten,
+    Divergent,
+    Abandoned,
+}
+
 pub struct MutableRepo {
     base_repo: Arc<ReadonlyRepo>,
     index: Box<dyn MutableIndex>,
     view: DirtyCell<View>,
-    rewritten_commits: HashMap<CommitId, HashSet<CommitId>>,
-    abandoned_commits: HashSet<CommitId>,
+    // The commit identified by the key has been replaced by all the ones in the value.
+    // * Branches pointing to the old commit should be updated to the new commit, resulting in a
+    //   conflict if there multiple new commits.
+    // * Children of the old commit should be rebased onto the new commits. However, if the type is
+    //   `Divergent`, they should be left in place.
+    // * Working copies pointing to the old commit should be updated to the first of the new
+    //   commits. However, if the type is `Abandoned`, a new working-copy commit should be created
+    //   on top of all of the new commits instead.
+    parent_mapping: HashMap<CommitId, (RewriteType, Vec<CommitId>)>,
 }
 
 impl MutableRepo {
@@ -748,8 +795,7 @@ impl MutableRepo {
             base_repo,
             index: mut_index,
             view: DirtyCell::with_clean(mut_view),
-            rewritten_commits: Default::default(),
-            abandoned_commits: Default::default(),
+            parent_mapping: Default::default(),
         }
     }
 
@@ -766,9 +812,7 @@ impl MutableRepo {
     }
 
     pub fn has_changes(&self) -> bool {
-        !(self.abandoned_commits.is_empty()
-            && self.rewritten_commits.is_empty()
-            && self.view() == &self.base_repo.view)
+        !(self.parent_mapping.is_empty() && self.view() == &self.base_repo.view)
     }
 
     pub(crate) fn consume(self) -> (Box<dyn MutableIndex>, View) {
@@ -808,49 +852,35 @@ impl MutableRepo {
         Ok(commit)
     }
 
-    /// Record a commit as having been rewritten to one or more ids in this
+    /// Record a commit as having been rewritten to another commit in this
     /// transaction.
     ///
     /// This record is used by `rebase_descendants` to know which commits have
     /// children that need to be rebased, and where to rebase them to. See the
     /// docstring for `record_rewritten_commit` for details.
-    //  TODO(ilyagr): Make this API saner, e.g. make `self.rewritten_commits` public
-    // and make empty values correspond to abandoned commits.
-    pub fn set_rewritten_commit(
+    pub fn set_rewritten_commit(&mut self, old_id: CommitId, new_id: CommitId) {
+        assert_ne!(old_id, *self.store().root_commit_id());
+        self.parent_mapping
+            .insert(old_id, (RewriteType::Rewritten, vec![new_id]));
+    }
+
+    /// Record a commit as being rewritten into multiple other commits in this
+    /// transaction.
+    ///
+    /// A later call to `rebase_descendants()` will update branches pointing to
+    /// `old_id` be conflicted and pointing to all pf `new_ids`. Working copies
+    /// pointing to `old_id` will be updated to point to the first commit in
+    /// `new_ids``. Descendants of `old_id` will be left alone.
+    pub fn set_divergent_rewrite(
         &mut self,
         old_id: CommitId,
         new_ids: impl IntoIterator<Item = CommitId>,
     ) {
         assert_ne!(old_id, *self.store().root_commit_id());
-        self.rewritten_commits
-            .insert(old_id, new_ids.into_iter().collect());
-    }
-
-    /// Record a commit as having been rewritten in this transaction. If it was
-    /// already rewritten, mark it as divergent (unlike `set_rewritten_commit`)
-    ///
-    /// This record is used by `rebase_descendants` to know which commits have
-    /// children that need to be rebased, and where to rebase the children (as
-    /// well as branches) to.
-    ///
-    /// The `rebase_descendants` logic treats these records as follows:
-    ///
-    /// - If a commit is recorded as rewritten to a single commit, its
-    ///   descendants would be rebased to become descendants of `new_id`. Any
-    ///   branches at `old_id` are also moved to `new_id`.
-    /// - If a commit is recorded as rewritten to more than one commit, it is
-    ///   assumed to have become divergent. Its descendants are *not* rebased.
-    ///   However, the *branches* that were at `old_id` are moved to each of the
-    ///   new ids, and thus become conflicted.
-    ///
-    /// In neither case would `rebase_descendants` modify the `old_id` commit
-    /// itself.
-    pub fn record_rewritten_commit(&mut self, old_id: CommitId, new_id: CommitId) {
-        assert_ne!(old_id, *self.store().root_commit_id());
-        self.rewritten_commits
-            .entry(old_id)
-            .or_default()
-            .insert(new_id);
+        self.parent_mapping.insert(
+            old_id.clone(),
+            (RewriteType::Divergent, new_ids.into_iter().collect()),
+        );
     }
 
     /// Record a commit as having been abandoned in this transaction.
@@ -862,37 +892,319 @@ impl MutableRepo {
     /// The `rebase_descendants` logic will rebase the descendants of `old_id`
     /// to become the descendants of parent(s) of `old_id`. Any branches at
     /// `old_id` would be moved to the parent(s) of `old_id` as well.
+    // TODO: Propagate errors from commit lookup or take a Commit as argument.
     pub fn record_abandoned_commit(&mut self, old_id: CommitId) {
         assert_ne!(old_id, *self.store().root_commit_id());
-        self.abandoned_commits.insert(old_id);
+        // Descendants should be rebased onto the commit's parents
+        let old_commit = self.store().get_commit(&old_id).unwrap();
+        self.record_abandoned_commit_with_parents(old_id, old_commit.parent_ids().to_vec());
     }
 
-    fn clear_descendant_rebaser_plans(&mut self) {
-        self.rewritten_commits.clear();
-        self.abandoned_commits.clear();
+    /// Record a commit as having been abandoned in this transaction.
+    ///
+    /// A later `rebase_descendants()` will rebase children of `old_id` onto
+    /// `new_parent_ids`. A working copy pointing to `old_id` will point to a
+    /// new commit on top of `new_parent_ids`.
+    pub fn record_abandoned_commit_with_parents(
+        &mut self,
+        old_id: CommitId,
+        new_parent_ids: impl IntoIterator<Item = CommitId>,
+    ) {
+        assert_ne!(old_id, *self.store().root_commit_id());
+        self.parent_mapping.insert(
+            old_id,
+            (RewriteType::Abandoned, new_parent_ids.into_iter().collect()),
+        );
     }
 
     pub fn has_rewrites(&self) -> bool {
-        !(self.rewritten_commits.is_empty() && self.abandoned_commits.is_empty())
+        !self.parent_mapping.is_empty()
+    }
+
+    /// Calculates new parents for a commit that's currently based on the given
+    /// parents. It does that by considering how previous commits have been
+    /// rewritten and abandoned.
+    ///
+    /// Panics if `parent_mapping` contains cycles
+    pub fn new_parents(&self, old_ids: Vec<CommitId>) -> Vec<CommitId> {
+        fn single_substitution_round(
+            parent_mapping: &HashMap<CommitId, (RewriteType, Vec<CommitId>)>,
+            ids: Vec<CommitId>,
+        ) -> (Vec<CommitId>, bool) {
+            let mut made_replacements = false;
+            let mut new_ids = vec![];
+            // TODO(ilyagr): (Maybe?) optimize common case of replacements all
+            // being singletons. If CommitId-s were Copy. no allocations would be needed in
+            // that case, but it probably doesn't matter much while they are Vec<u8>-s.
+            for id in ids.into_iter() {
+                match parent_mapping.get(&id) {
+                    None | Some((RewriteType::Divergent, _)) => {
+                        new_ids.push(id);
+                    }
+                    Some((_, replacements)) => {
+                        assert!(
+                            // Each commit must have a parent, so a parent can
+                            // not just be mapped to nothing. This assertion
+                            // could be removed if this function is used for
+                            // mapping something other than a commit's parents.
+                            !replacements.is_empty(),
+                            "Found empty value for key {id:?} in the parent mapping",
+                        );
+                        made_replacements = true;
+                        new_ids.extend(replacements.iter().cloned())
+                    }
+                };
+            }
+            (new_ids, made_replacements)
+        }
+
+        let mut new_ids = old_ids;
+        // The longest possible non-cycle substitution sequence goes through each key of
+        // parent_mapping once.
+        let mut allowed_iterations = 0..self.parent_mapping.len();
+        loop {
+            let made_replacements;
+            (new_ids, made_replacements) = single_substitution_round(&self.parent_mapping, new_ids);
+            if !made_replacements {
+                break;
+            }
+            allowed_iterations
+                .next()
+                .expect("cycle detected in the parent mapping");
+        }
+        match new_ids.as_slice() {
+            // The first two cases are an optimization for the common case of commits with <=2
+            // parents
+            [_singleton] => new_ids,
+            [a, b] if a != b => new_ids,
+            _ => new_ids.into_iter().unique().collect(),
+        }
+    }
+
+    /// Updates branches, working copies, and anonymous heads after rewriting
+    /// and/or abandoning commits.
+    pub fn update_rewritten_references(&mut self, settings: &UserSettings) -> BackendResult<()> {
+        self.update_all_references(settings)?;
+        self.update_heads();
+        Ok(())
+    }
+
+    fn update_all_references(&mut self, settings: &UserSettings) -> Result<(), BackendError> {
+        for (old_parent_id, (_, new_parent_ids)) in self.parent_mapping.clone() {
+            // Call `new_parents()` here since `parent_mapping` only contains direct
+            // mappings, not transitive ones.
+            // TODO: keep parent_mapping updated with transitive mappings so we don't need
+            // to call `new_parents()` here.
+            let new_parent_ids = self.new_parents(new_parent_ids);
+            self.update_references(settings, old_parent_id, new_parent_ids)?;
+        }
+        Ok(())
+    }
+
+    fn update_references(
+        &mut self,
+        settings: &UserSettings,
+        old_commit_id: CommitId,
+        new_commit_ids: Vec<CommitId>,
+    ) -> Result<(), BackendError> {
+        // We arbitrarily pick a new working-copy commit among the candidates.
+        let abandoned_old_commit = matches!(
+            self.parent_mapping.get(&old_commit_id),
+            Some((RewriteType::Abandoned, _))
+        );
+        self.update_wc_commits(
+            settings,
+            &old_commit_id,
+            &new_commit_ids[0],
+            abandoned_old_commit,
+        )?;
+
+        // Build a map from commit to branches pointing to it, so we don't need to scan
+        // all branches each time we rebase a commit.
+        // TODO: We no longer need to do this now that we update branches for all
+        // commits at once.
+        let mut branches: HashMap<_, HashSet<_>> = HashMap::new();
+        for (branch_name, target) in self.view().local_branches() {
+            for commit in target.added_ids() {
+                branches
+                    .entry(commit.clone())
+                    .or_default()
+                    .insert(branch_name.to_owned());
+            }
+        }
+
+        if let Some(branch_names) = branches.get(&old_commit_id).cloned() {
+            let mut branch_updates = vec![];
+            for branch_name in &branch_names {
+                let local_target = self.get_local_branch(branch_name);
+                for old_add in local_target.added_ids() {
+                    if *old_add == old_commit_id {
+                        branch_updates.push(branch_name.clone());
+                    }
+                }
+            }
+
+            let old_target = RefTarget::normal(old_commit_id.clone());
+            assert!(!new_commit_ids.is_empty());
+            let new_target = RefTarget::from_legacy_form(
+                std::iter::repeat(old_commit_id).take(new_commit_ids.len() - 1),
+                new_commit_ids,
+            );
+            for branch_name in &branch_updates {
+                self.merge_local_branch(branch_name, &old_target, &new_target);
+            }
+        }
+
+        Ok(())
+    }
+
+    fn update_wc_commits(
+        &mut self,
+        settings: &UserSettings,
+        old_commit_id: &CommitId,
+        new_commit_id: &CommitId,
+        abandoned_old_commit: bool,
+    ) -> Result<(), BackendError> {
+        let workspaces_to_update = self.view().workspaces_for_wc_commit_id(old_commit_id);
+        if workspaces_to_update.is_empty() {
+            return Ok(());
+        }
+
+        let new_commit = self.store().get_commit(new_commit_id)?;
+        let new_wc_commit = if !abandoned_old_commit {
+            new_commit
+        } else {
+            self.new_commit(
+                settings,
+                vec![new_commit.id().clone()],
+                new_commit.tree_id().clone(),
+            )
+            .write()?
+        };
+        for workspace_id in workspaces_to_update.into_iter() {
+            self.edit(workspace_id, &new_wc_commit).unwrap();
+        }
+        Ok(())
+    }
+
+    fn update_heads(&mut self) {
+        let old_commits_expression =
+            RevsetExpression::commits(self.parent_mapping.keys().cloned().collect());
+        let heads_to_add_expression = old_commits_expression
+            .parents()
+            .minus(&old_commits_expression);
+        let heads_to_add = heads_to_add_expression
+            .evaluate_programmatic(self)
+            .unwrap()
+            .iter();
+
+        let mut view = self.view().store_view().clone();
+        for commit_id in self.parent_mapping.keys() {
+            view.head_ids.remove(commit_id);
+        }
+        view.head_ids.extend(heads_to_add);
+        self.set_view(view);
+    }
+
+    /// Find descendants of `root`, unless they've already been rewritten
+    /// (according to `parent_mapping`), and then return them in
+    /// an order they should be rebased in. The result is in reverse order
+    /// so the next value can be removed from the end.
+    fn find_descendants_to_rebase(&self, roots: Vec<CommitId>) -> BackendResult<Vec<Commit>> {
+        let store = self.store();
+        let to_visit_expression =
+            RevsetExpression::commits(roots)
+                .descendants()
+                .minus(&RevsetExpression::commits(
+                    self.parent_mapping.keys().cloned().collect(),
+                ));
+        let to_visit_revset = to_visit_expression
+            .evaluate_programmatic(self)
+            .map_err(|err| match err {
+                RevsetEvaluationError::StoreError(err) => err,
+                RevsetEvaluationError::Other(_) => panic!("Unexpected revset error: {err}"),
+            })?;
+        let to_visit: Vec<_> = to_visit_revset.iter().commits(store).try_collect()?;
+        drop(to_visit_revset);
+        let to_visit_set: HashSet<CommitId> =
+            to_visit.iter().map(|commit| commit.id().clone()).collect();
+        let mut visited = HashSet::new();
+        // Calculate an order where we rebase parents first, but if the parents were
+        // rewritten, make sure we rebase the rewritten parent first.
+        dag_walk::topo_order_reverse_ok(
+            to_visit.into_iter().map(Ok),
+            |commit| commit.id().clone(),
+            |commit| -> Vec<BackendResult<Commit>> {
+                visited.insert(commit.id().clone());
+                let mut dependents = vec![];
+                for parent in commit.parents() {
+                    if let Some((_, targets)) = self.parent_mapping.get(parent.id()) {
+                        for target in targets {
+                            if to_visit_set.contains(target) && !visited.contains(target) {
+                                dependents.push(store.get_commit(target));
+                            }
+                        }
+                    }
+                    if to_visit_set.contains(parent.id()) {
+                        dependents.push(Ok(parent));
+                    }
+                }
+                dependents
+            },
+        )
+    }
+
+    /// Rewrite descendants of the given roots.
+    ///
+    /// The callback will be called for each commit with the new parents
+    /// prepopulated. The callback may change the parents and write the new
+    /// commit, or it may abandon the commit, or it may leave the old commit
+    /// unchanged.
+    ///
+    /// The set of commits to visit is determined at the start. If the callback
+    /// adds new descendants, then the callback will not be called for those.
+    /// Similarly, if the callback rewrites unrelated commits, then the callback
+    /// will not be called for descendants of those commits.
+    pub fn transform_descendants(
+        &mut self,
+        settings: &UserSettings,
+        roots: Vec<CommitId>,
+        mut callback: impl FnMut(CommitRewriter) -> BackendResult<()>,
+    ) -> BackendResult<()> {
+        let mut to_visit = self.find_descendants_to_rebase(roots)?;
+        while let Some(old_commit) = to_visit.pop() {
+            let new_parent_ids = self.new_parents(old_commit.parent_ids().to_vec());
+            let rewriter = CommitRewriter::new(self, old_commit, new_parent_ids);
+            callback(rewriter)?;
+        }
+        self.update_rewritten_references(settings)?;
+        // Since we didn't necessarily visit all descendants of rewritten commits (e.g.
+        // if they were rewritten in the callback), there can still be commits left to
+        // rebase, so we don't clear `parent_mapping` here.
+        // TODO: Should we make this stricter? We could check that there were no
+        // rewrites before this function was called, and we can check that only
+        // commits in the `to_visit` set were added by the callback. Then we
+        // could clear `parent_mapping` here and not have to scan it again at
+        // the end of the transaction when we call `rebase_descendants()`.
+
+        Ok(())
     }
 
     /// After the rebaser returned by this function is dropped,
-    /// self.clear_descendant_rebaser_plans() needs to be called.
+    /// self.parent_mapping needs to be cleared.
     fn rebase_descendants_return_rebaser<'settings, 'repo>(
         &'repo mut self,
         settings: &'settings UserSettings,
         options: RebaseOptions,
-    ) -> Result<Option<DescendantRebaser<'settings, 'repo>>, TreeMergeError> {
+    ) -> BackendResult<Option<DescendantRebaser<'settings, 'repo>>> {
         if !self.has_rewrites() {
             // Optimization
             return Ok(None);
         }
-        let mut rebaser = DescendantRebaser::new(
-            settings,
-            self,
-            self.rewritten_commits.clone(),
-            self.abandoned_commits.clone(),
-        );
+
+        let to_visit =
+            self.find_descendants_to_rebase(self.parent_mapping.keys().cloned().collect())?;
+        let mut rebaser = DescendantRebaser::new(settings, self, to_visit);
         *rebaser.mut_options() = options;
         rebaser.rebase_all()?;
         Ok(Some(rebaser))
@@ -905,11 +1217,11 @@ impl MutableRepo {
         &mut self,
         settings: &UserSettings,
         options: RebaseOptions,
-    ) -> Result<usize, TreeMergeError> {
+    ) -> BackendResult<usize> {
         let result = self
             .rebase_descendants_return_rebaser(settings, options)?
             .map_or(0, |rebaser| rebaser.into_map().len());
-        self.clear_descendant_rebaser_plans();
+        self.parent_mapping.clear();
         Ok(result)
     }
 
@@ -926,29 +1238,41 @@ impl MutableRepo {
     /// **its parent**. One can tell this case apart since the change ids of the
     /// key and the value will not match. The parent will inherit the
     /// descendants and the branches of the abandoned commit.
+    // TODO: Rewrite this using `transform_descendants()`
     pub fn rebase_descendants_with_options_return_map(
         &mut self,
         settings: &UserSettings,
         options: RebaseOptions,
-    ) -> Result<HashMap<CommitId, CommitId>, TreeMergeError> {
+    ) -> BackendResult<HashMap<CommitId, CommitId>> {
         let result = Ok(self
             // We do not set RebaseOptions here, since this function does not currently return
             // enough information to describe the results of a rebase if some commits got
             // abandoned
             .rebase_descendants_return_rebaser(settings, options)?
             .map_or(HashMap::new(), |rebaser| rebaser.into_map()));
-        self.clear_descendant_rebaser_plans();
+        self.parent_mapping.clear();
         result
     }
 
-    pub fn rebase_descendants(&mut self, settings: &UserSettings) -> Result<usize, TreeMergeError> {
-        self.rebase_descendants_with_options(settings, Default::default())
+    pub fn rebase_descendants(&mut self, settings: &UserSettings) -> BackendResult<usize> {
+        let roots = self.parent_mapping.keys().cloned().collect_vec();
+        let mut num_rebased = 0;
+        self.transform_descendants(settings, roots, |rewriter| {
+            if rewriter.parents_changed() {
+                let builder = rewriter.rebase(settings)?;
+                builder.write()?;
+                num_rebased += 1;
+            }
+            Ok(())
+        })?;
+        self.parent_mapping.clear();
+        Ok(num_rebased)
     }
 
     pub fn rebase_descendants_return_map(
         &mut self,
         settings: &UserSettings,
-    ) -> Result<HashMap<CommitId, CommitId>, TreeMergeError> {
+    ) -> BackendResult<HashMap<CommitId, CommitId>> {
         self.rebase_descendants_with_options_return_map(settings, Default::default())
     }
 
@@ -1345,8 +1669,13 @@ impl MutableRepo {
             rewritten_changes.insert(change_id);
         }
         for (old_commit, new_commits) in rewritten_commits {
-            for new_commit in new_commits {
-                self.record_rewritten_commit(old_commit.clone(), new_commit);
+            if new_commits.len() == 1 {
+                self.set_rewritten_commit(
+                    old_commit.clone(),
+                    new_commits.into_iter().next().unwrap(),
+                );
+            } else {
+                self.set_divergent_rewrite(old_commit.clone(), new_commits);
             }
         }
 
